//#preprocessor

/*
 * File: ICC_Converter.java
 * Version: 2.0
 * Initial Creation: Jun 29, 2010 11:53:24 PM
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * SEE "Little CMS License.txt"
 */
package com.sun.pdfview.helper.graphics.color;

//#ifndef USE_LCMS_LIBRARY
import net.rim.device.api.util.Arrays;
//#endif

//Based off LCMS 2.0, Apache-Harmony 6.0

/**
 * ICC color profile converter. Elements taken from Apache's work on J2SE's ICC color system.
 * @author Vincent Simonetti
 */
public class ICC_Converter
{
//#ifndef USE_LCMS_LIBRARY
	private int inputFormat, outputFormat; // Keep formats for further reference
	
	// Points to transform code
    private TransformOp xform;
    
    // Formatters, cannot be embedded into LUT because cache
    private Formatter16 fromInput;
    private Formatter16 toOutput;

    private FormatterFloat fromInputFloat;
    private FormatterFloat toOutputFloat;
    
    // 1-pixel cache (16 bits only)
    private short[] cacheIn, cacheOut;
	
	// A MPE LUT holding the full (optimized) transform
    private ICC_Pipeline lut;
    
    // A MPE LUT holding the gamut check. It goes from the input space to bilevel
    private ICC_Pipeline gamutCheck;
//#endif
	// Informational only
    private int entryColorSpace, exitColorSpace;
//#ifndef USE_LCMS_LIBRARY
    private int originalFlags;
    
    private static FormatterItem[] inputFormatters;
    private static FormatterItem[] outputFormatters;
    
    private static final int ANYSPACE = 31 << CMM.COLORSPACE_SH;
    
    static
    {
    	inputFormatters = new FormatterItem[1];
    	outputFormatters = new FormatterItem[1];
    	
    	//Inputs
    	inputFormatters[0] = new Formatter16Item((3 << CMM.CHANNELS_SH) | (2 << CMM.BYTES_SH), ANYSPACE, new Formatter16()
    	{
			public Object format(short[] values, Object buffer, int[] offset, int offsetIndex, int stride)
			{
				//Unroll3Words
				if(buffer instanceof short[])
				{
					short[] accumS = (short[])buffer;
					values[0] = accumS[offset[offsetIndex]++]; // C R
					values[1] = accumS[offset[offsetIndex]++]; // M G
					values[2] = accumS[offset[offsetIndex]++]; // Y B
				}
				else
				{
					System.err.println("Unsupported buffer format in InputFormatter. Expected short[], got " + buffer.getClass().getName());
				}
				return buffer;
			}
		});
    	
    	//Outputs
    	outputFormatters[0] = new Formatter16Item((3 << CMM.CHANNELS_SH) | (2 << CMM.BYTES_SH), ANYSPACE, new Formatter16()
    	{
			public Object format(short[] values, Object buffer, int[] offset, int offsetIndex, int stride)
			{
				//Pack3Words
				if(buffer instanceof short[])
				{
					short[] accumS = (short[])buffer;
					accumS[offset[offsetIndex]++] = values[0];
					accumS[offset[offsetIndex]++] = values[1];
					accumS[offset[offsetIndex]++] = values[2];
				}
				else
				{
					System.err.println("Unsupported buffer format in OutputFormatter. Expected short[], got " + buffer.getClass().getName());
				}
				return buffer;
			}
		});
    }
//#else
    
    public ICC_Converter(ICC_Profile[] profiles, int[] intents)
    {
    	//TODO
    }
//#endif
    
//#ifndef USE_LCMS_LIBRARY
    /*
	private ICC_Converter(ICC_Profile input, int inputFormat, ICC_Profile output, int outputFormat, int intent, int flags)
	{
		this(new ICC_Profile[]{input, output}, inputFormat, outputFormat, intent, flags);
	}
	
	private ICC_Converter(ICC_Profile[] profiles, int inputFormat, int outputFormat, int intent, int flags)
	{
		int nProfiles = profiles.length;
		if (nProfiles <= 0 || nProfiles > 255)
		{
			throw new IllegalArgumentException("Wrong number of profiles. 1..255 expected, " + nProfiles + " found.");
		}
		
		boolean[] bpc = new boolean[nProfiles];
		int[] intents = new int[nProfiles];
		double[] adaptationStates = new double[nProfiles];
		
		for (int i = 0; i < nProfiles; i++)
		{
			bpc[i] = (flags & CMM.cmsFLAGS_BLACKPOINTCOMPENSATION) != 0;
	        intents[i] = intent;
	        adaptationStates[i] = 0;
	    }
		
		createExtendedTransform(profiles, bpc, intents, adaptationStates, null, 0, inputFormat, outputFormat, flags);
	}
	*/
	
	private ICC_Converter(ICC_Profile[] profiles, boolean[] bpc, int[] intents, double[] adaptationStates, ICC_Profile gamutProfile, int gamutPCSposition, int inputFormat, int outputFormat, int flags)
	{
		cmsCreateExtendedTransform(profiles, bpc, intents, adaptationStates, gamutProfile, gamutPCSposition, inputFormat, outputFormat, flags);
	}
	
	private void cmsCreateExtendedTransform(ICC_Profile[] profiles, boolean[] bpc, int[] intents, double[] adaptationStates, ICC_Profile gamutProfile, int gamutPCSposition, int inputFormat, int outputFormat, int flags)
	{
		// If gamut check is requested, make sure we have a gamut profile
	    if ((flags & CMM.cmsFLAGS_GAMUTCHECK) != 0)
	    {
	        if (gamutProfile == null)
	        {
	        	flags &= ~CMM.cmsFLAGS_GAMUTCHECK;
	        }
	    }
	    
	    // On floating point transforms, inhibit optimizations 
	    boolean floatTransform = (_cmsFormatterIsFloat(inputFormat) && _cmsFormatterIsFloat(outputFormat));
	    
	    if (_cmsFormatterIsFloat(inputFormat) || _cmsFormatterIsFloat(outputFormat))
	    {
	    	flags |= CMM.cmsFLAGS_NOCACHE;
	    }
	    
	    // Mark entry/exit spaces
		if (!getXFormColorSpaces(profiles))
		{
			throw new NullPointerException("NULL input profiles on transform");
		}
		
		// Check if proper colorspaces
	    if (!isProperColorSpace(this.entryColorSpace, inputFormat))
	    {
	    	throw new IllegalArgumentException("Wrong input color space on transform");
	    }

	    if (!isProperColorSpace(this.exitColorSpace, outputFormat))
	    {
	    	throw new IllegalArgumentException("Wrong output color space on transform");
	    }
	    
	    // Create a pipeline with all transformations
	    ICC_Pipeline lut = ICC_Pipeline.cmsLinkProfiles(intents, profiles, bpc, adaptationStates, flags);
	    if (lut == null)
	    {
	    	throw new IllegalArgumentException("Couldn't link the profiles");
	    }
	    
	    // Optimize the LUT if possible
	    int[] flagsI = new int[]{flags}; //Flags can get changed
	    lut._cmsOptimizePipeline(intents[profiles.length - 1], inputFormat, outputFormat, flagsI);
	    flags = flagsI[0];
		
	    //AllocEmptyTransform is integrated into this function
	    //BEGIN AllocEmptyTransform
	    // Check whatever this is a true floating point transform
	    if (_cmsFormatterIsFloat(inputFormat) && _cmsFormatterIsFloat(outputFormat))
	    {
	        // Get formatter function always return a valid union, but the contents of this union may be NULL.
	    	this.fromInputFloat = (FormatterFloat)_cmsGetFormatter(inputFormat, true, true);//[0];
	    	this.toOutputFloat = (FormatterFloat)_cmsGetFormatter(outputFormat, false, true);//[0];
	        
	        if(this.fromInputFloat == null || this.toOutputFloat == null)
	        {
	        	throw new UnsupportedOperationException("Unsupported raster format");
	        }
	        
	        // Float transforms don't use cach, always are non-NULL
	        this.xform = new TransformOp()
	        {
	        	//FloatXFORM
				public void transform(Object inputBuffer, Object outputBuffer, int size)
				{
					// Float xform converts floats. Since there are no performance issues, one routine does all job, including gamut check.
					// Note that because extended range, we can use a -1.0 value for out of gamut in this case.
					
					float[] fIn = new float[ICC_Converter.this.getInputChannelCount()];
					float[] fOut = new float[ICC_Converter.this.getOutputChannelCount()];
					float[] outOfGamut = null;
					int[] offset = new int[2];
					
					for (int i = 0; i < size; i++)
					{
						inputBuffer = ICC_Converter.this.fromInputFloat.format(fIn, inputBuffer, offset, 0, size);
						
						// Any gamut chack to do?
				        if (ICC_Converter.this.gamutCheck != null)
				        {
				        	if(outOfGamut == null)
				        	{
				        		outOfGamut = new float[1];
				        	}
				            // Evaluate gamut marker.
				        	ICC_Converter.this.gamutCheck.evalFloatFn.evaluate(fIn, outOfGamut);
				        	
				            // Is current color out of gamut?
				            if (outOfGamut[0] > 0)
				            {
				                // Certainly, out of gamut
				            	int len = fOut.length;
				                for (int j = 0; j < len; j++)
				                {
				                	fOut[j] = -1;
				                }
				            }
				            else
				            {
				                // No, proceed normally
				            	ICC_Converter.this.lut.evalFloatFn.evaluate(fIn, fOut);
				            }
				        }
				        else
				        {
				            // No gamut check at all
				        	ICC_Converter.this.lut.evalFloatFn.evaluate(fIn, fOut);  
				        }
				        
				        // Back to asked representation
				        outputBuffer = ICC_Converter.this.toOutputFloat.format(fOut, outputBuffer, offset, 1, size);
					}
				}
			};
	    }
	    else
	    {
	    	this.fromInput = (Formatter16)_cmsGetFormatter(inputFormat, true, false);//[1];
	    	this.toOutput = (Formatter16)_cmsGetFormatter(outputFormat, false, false);//[1];
	        
	        if(this.fromInput == null || this.toOutput == null)
	        {
	        	throw new UnsupportedOperationException("Unsupported raster format");
	        }
	        
	        if((flags & CMM.cmsFLAGS_NULLTRANSFORM) != 0)
	        {
	        	this.xform = new TransformOp()
	        	{
	        		//NullXFORM
					public void transform(Object inputBuffer, Object outputBuffer, int size)
					{
						// Null transformation, only applies formatters. No cach
						
						short[] wIn = new short[ICC_Converter.this.getInputChannelCount()];
						int[] offset = new int[2];
						
						for (int i = 0; i < size; i++)
						{
							inputBuffer = ICC_Converter.this.fromInput.format(wIn, inputBuffer, offset, 0, size);
							outputBuffer = ICC_Converter.this.toOutput.format(wIn, outputBuffer, offset, 1, size);
					    }
					}
				};
	        }
	        else
	        {
	            if ((flags & CMM.cmsFLAGS_NOCACHE) != 0)
	            {
	                if ((flags & CMM.cmsFLAGS_GAMUTCHECK) != 0)
	                {
	                	this.xform = new TransformOp()
	                	{
	                		//PrecalculatedXFORMGamutCheck
							public void transform(Object inputBuffer, Object outputBuffer, int size) // Gamut check, no cach
							{
								// Gamut check, No cach, 16 bits.
								
								short[] wIn = new short[ICC_Converter.this.getInputChannelCount()];
								short[] wOut = new short[ICC_Converter.this.getOutputChannelCount()];
								int[] offset = new int[2];
								
								for (int i = 0; i < size; i++)
								{
									inputBuffer = ICC_Converter.this.fromInput.format(wIn, inputBuffer, offset, 0, size);
									ICC_Converter.transformOnePixelWithGamutCheck(ICC_Converter.this, wIn, wOut);
							        outputBuffer = ICC_Converter.this.toOutput.format(wOut, outputBuffer, offset, 1, size);
							    }
							}
						};
	                }
	                else
	                {
	                	this.xform = new TransformOp()
	                	{
	                		//PrecalculatedXFORM
							public void transform(Object inputBuffer, Object outputBuffer, int size) // No cach, no gamut check
							{
								// No gamut check, no cache, 16 bits
								
								short[] wIn = new short[ICC_Converter.this.getInputChannelCount()];
								short[] wOut = new short[ICC_Converter.this.getOutputChannelCount()];
								int[] offset = new int[2];
								
								for (int i = 0; i < size; i++)
								{
									inputBuffer = ICC_Converter.this.fromInput.format(wIn, inputBuffer, offset, 0, size);
									ICC_Converter.this.lut.eval16Fn.evaluate(wIn, wOut);
							        outputBuffer = ICC_Converter.this.toOutput.format(wOut, outputBuffer, offset, 1, size);
							    }
							}
						};
	                }
	            }
	            else
	            {
	            	if ((flags & CMM.cmsFLAGS_GAMUTCHECK) != 0)
	            	{
	            		this.xform = new TransformOp()
	            		{
	            			//CachedXFORMGamutCheck
							public void transform(Object inputBuffer, Object outputBuffer, int size) // Gamut check, cach
							{
								// All those nice features together
								
								short[] wIn = new short[ICC_Converter.this.getInputChannelCount()];
								short[] wOut = new short[ICC_Converter.this.getOutputChannelCount()];
								short[] cacheIn, cacheOut;
								int[] offset = new int[2];
								int cacheInLen, cacheOutLen;
								
								synchronized(ICC_Converter.this.cacheIn)
								{
									cacheIn = Arrays.copy(ICC_Converter.this.cacheIn);
									cacheInLen = cacheIn.length;
								}
								synchronized(ICC_Converter.this.cacheOut)
								{
									cacheOut = Arrays.copy(ICC_Converter.this.cacheOut);
									cacheOutLen = cacheOut.length;
								}
								
								for(int i = 0; i < size; i++)
								{
									inputBuffer = ICC_Converter.this.fromInput.format(wIn, inputBuffer, offset, 0, size);
									
									if(Arrays.equals(wIn, cacheIn))
									{
										System.arraycopy(wOut, 0, cacheOut, 0, cacheOutLen);
									}
									else
									{
										transformOnePixelWithGamutCheck(ICC_Converter.this, wIn, wOut);
										
										System.arraycopy(wIn, 0, cacheIn, 0, cacheInLen);
										System.arraycopy(wOut, 0, cacheOut, 0, cacheOutLen);
									}
									
									outputBuffer = ICC_Converter.this.toOutput.format(wOut, outputBuffer, offset, 1, size);
								}
								
								synchronized(ICC_Converter.this.cacheIn)
								{
									System.arraycopy(cacheIn, 0, ICC_Converter.this.cacheIn, 0, cacheInLen);
								}
								synchronized(ICC_Converter.this.cacheOut)
								{
									System.arraycopy(cacheOut, 0, ICC_Converter.this.cacheOut, 0, cacheOutLen);
								}
							}
						};
	            	}
	                else
	                {
	                	this.xform = new TransformOp()
	                	{
	                		//CachedXFORM
							public void transform(Object inputBuffer, Object outputBuffer, int size) // No gamut check, cach
							{
								// No gamut check, Cach, 16 bits, 
								
								short[] wIn = new short[ICC_Converter.this.getInputChannelCount()];
								short[] wOut = new short[ICC_Converter.this.getOutputChannelCount()];
								short[] cacheIn, cacheOut;
								int[] offset = new int[2];
								int cacheInLen, cacheOutLen;
								
								synchronized(ICC_Converter.this.cacheIn)
								{
									cacheIn = Arrays.copy(ICC_Converter.this.cacheIn);
									cacheInLen = cacheIn.length;
								}
								synchronized(ICC_Converter.this.cacheOut)
								{
									cacheOut = Arrays.copy(ICC_Converter.this.cacheOut);
									cacheOutLen = cacheOut.length;
								}
								
								for(int i = 0; i < size; i++)
								{
									inputBuffer = ICC_Converter.this.fromInput.format(wIn, inputBuffer, offset, 0, size);
									
									if(Arrays.equals(wIn, cacheIn))
									{
										System.arraycopy(wOut, 0, cacheOut, 0, cacheOutLen);
									}
									else
									{
										ICC_Converter.this.lut.eval16Fn.evaluate(wIn, wOut);
										
										System.arraycopy(wIn, 0, cacheIn, 0, cacheInLen);
										System.arraycopy(wOut, 0, cacheOut, 0, cacheOutLen);
									}
									
									outputBuffer = ICC_Converter.this.toOutput.format(wOut, outputBuffer, offset, 1, size);
								}
								
								synchronized(ICC_Converter.this.cacheIn)
								{
									System.arraycopy(cacheIn, 0, ICC_Converter.this.cacheIn, 0, cacheInLen);
								}
								synchronized(ICC_Converter.this.cacheOut)
								{
									System.arraycopy(cacheOut, 0, ICC_Converter.this.cacheOut, 0, cacheOutLen);
								}
							}
						};
	                }
	            }
	        }
	    }
	    
	    this.inputFormat = inputFormat;
	    this.outputFormat = outputFormat;
	    this.originalFlags = flags;
	    //END AllocEmptyTransform
	    
	    // Keep values
	    this.lut = lut;
	    
	    // Create a gamut check LUT if requested
	    if (gamutProfile != null && ((flags & CMM.cmsFLAGS_GAMUTCHECK) != 0))
	    {
	    	this.gamutCheck = _cmsCreateGamutCheckPipeline(profiles, bpc, intents, adaptationStates, gamutPCSposition, gamutProfile);
	    }
	    
	    // Output is a little bit more complex.
	    ICC_Profile profile = profiles[profiles.length - 1];
	    if (CMM.cmsGetDeviceClass(profile) == ICC_Profile.icSigLinkClass)
	    {
	        // This tag may exist only on devicelink profiles.
	    	if (profile.cmm.cmsIsTag(ICC_Profile.icSigColorantTableOutTag))
	    	{
	    		// It may be NULL if error
	    		//XXX cmsxform.c:#580
	    		throw new UnsupportedOperationException("ICC_Converter.cmsCreateExtendedTransform#link/colorantTableOutTag not implemented");
	            //this.outputColorant = cmsDupNamedColorList(profile.cmm.cmsReadTag(ICC_Profile.icSigColorantTableOutTag));
	        }
	    }
	    else
	    {
	    	if (profile.cmm.cmsIsTag(ICC_Profile.icSigColorantTableTag))
	    	{
	    		//XXX cmsxform.c:#587
	    		throw new UnsupportedOperationException("ICC_Converter.cmsCreateExtendedTransform#colorantTableTag not implemented");
	    		//this.outputColorant = cmsDupNamedColorList(profile.cmm.cmsReadTag(ICC_Profile.icSigColorantTableTag));
	        }     
	    }
	    
	    /* Store the sequence of profiles
	     * --Not sure if this is required anywhere so skip it
	    if ((flags & CMM.cmsFLAGS_KEEP_SEQUENCE) != 0)
	    {
	        this.sequence = _cmsCompileProfileSequence(profiles);
	    }
	    else
	    {
	    	this.sequence = null;
	    }
	    */
	    
	    // If this is a cached transform, init first value, which is zero (16 bits only)
	    if ((flags & CMM.cmsFLAGS_NOCACHE) == 0)
	    {
	        this.cacheIn = new short[this.getInputChannelCount()];
	        this.cacheOut = new short[this.getOutputChannelCount()];
	        
	        if (this.gamutCheck != null)
	        {
	            transformOnePixelWithGamutCheck(this, this.cacheIn, this.cacheOut);
	        }
	        else
	        {
	        	this.lut.eval16Fn.evaluate(this.cacheIn, this.cacheOut);
	        }
	    }
	}
//#endif
	public int getInputChannelCount()
	{
		return CMM.cmsChannelsOf(this.entryColorSpace);
	}
	
	public int getOutputChannelCount()
	{
		return CMM.cmsChannelsOf(this.exitColorSpace);
	}
//#ifndef USE_LCMS_LIBRARY
	private boolean getXFormColorSpaces(ICC_Profile[] profiles)
	{
		if(profiles == null || profiles.length == 0 || profiles[0] == null)
		{
			return false;
		}
		
		byte[] header;
		int colorSpaceIn, colorSpaceOut, postColorSpace; //What is colorSpaceIn used for?
		this.entryColorSpace = postColorSpace = CMM.getIntFromByteArray(header = profiles[0].getData(ICC_Profile.icSigHead), ICC_Profile.icHdrColorSpace);
		
		int len = profiles.length;
		for (int i = 0; i < len; i++)
		{
			ICC_Profile profile = profiles[i];
	        
			if (profile == null)
			{
				return false;
			}
			
			if(i != 0)
			{
				header = profiles[i].getData(ICC_Profile.icSigHead);
			}
			
	        if ((postColorSpace != ICC_Profile.icSigXYZData) && (postColorSpace != ICC_Profile.icSigLabData))
	        {
	        	colorSpaceIn = CMM.getIntFromByteArray(header, ICC_Profile.icHdrColorSpace);
	            colorSpaceOut = CMM.getIntFromByteArray(header, ICC_Profile.icHdrPcs);
	        }
	        else
	        {
	        	colorSpaceIn = CMM.getIntFromByteArray(header, ICC_Profile.icHdrPcs);
	            colorSpaceOut = CMM.getIntFromByteArray(header, ICC_Profile.icHdrColorSpace);
	        }
	        
	        postColorSpace = colorSpaceOut;
	    }

	    this.exitColorSpace = postColorSpace;

		return true;
	}
	
	// Does compute a gamut LUT going back and forth across pcs -> relativ. colorimetric intent -> pcs
	// the dE obtained is then annotated on the LUT. Values truely out of gamut are clipped to dE = 0xFFFE
	// and values changed are supposed to be handled by any gamut remapping, so, are out of gamut as well.
	//
	// **WARNING: This algorithm does assume that gamut remapping algorithms does NOT move in-gamut colors,
	// of course, many perceptual and saturation intents does not work in such way, but relativ. ones should.

	private static ICC_Pipeline _cmsCreateGamutCheckPipeline(ICC_Profile[] profiles, boolean[] bpc, int[] intents, double[] adaptationStates, int gamutPCSposition, 
			ICC_Profile gamut)
	{
		//XXX _cmsCreateGamutCheckPipeline:cmsgmt.c:#291
		throw new UnsupportedOperationException("ICC_Converter._cmsCreateGamutCheckPipeline not implemented");
	}
	
	// Auxiliar: Handle precalculated gamut check
	private static void transformOnePixelWithGamutCheck(ICC_Converter p, short[] wIn, short[] wOut)
	{
		short[] wOutOfGamut = new short[1];
		
		p.gamutCheck.eval16Fn.evaluate(wIn, wOutOfGamut);
		
		if(wOutOfGamut[0] >= 1)
		{
			for(int i = 0; i < p.lut.outputChannels; i++)
			{
				wOut[i] = -1; //Don't have alarm codes and they aren't used anyway so this should never get called
			}
	    }
	    else
	    {
	    	p.lut.eval16Fn.evaluate(wIn, wOut);
	    } 
	}
	
	private static Object _cmsGetFormatter(int type, // Specific type, i.e. TYPE_RGB_8
            boolean dir, 
            boolean flags) // Float or 16 bits
	{
		//Original code checks the "FactoryList" which is setup when plug-ins get used.
		
		return getFormmaterItem(type, dir ? inputFormatters : outputFormatters, flags);
	}
	
	private static Object getFormmaterItem(int input, FormatterItem[] list, boolean fp)
	{
		int len = list.length;
		for(int i = 0; i < len; i++)
		{
			FormatterItem item = list[i];
			if ((input & ~item.mask) == item.type)
			{
				if(fp)
				{
					if(item instanceof FormatterFloatItem)
					{
						return ((FormatterFloatItem)item).frm;
					}
				}
				else
				{
					if(item instanceof Formatter16Item)
					{
						return ((Formatter16Item)item).frm;
					}
				}
			}
		}
		return null;
	}
	
	// Check colorspace
	private static boolean isProperColorSpace(int check, int format)
	{
	    int space1 = (format >> CMM.COLORSPACE_SH) & 0x1F;
	    int space2 = CMM.cmsLCMScolorSpace(check);
	    
	    if (space1 == CMM.PT_ANY)
	    {
	    	return true;
	    }
	    if (space1 == space2)
	    {
	    	return true;
	    }
	    
	    if (space1 == CMM.PT_LabV2 && space2 == CMM.PT_Lab)
	    {
	    	return true;
	    }
	    if (space1 == CMM.PT_Lab && space2 == CMM.PT_LabV2)
	    {
	    	return true;
	    }
	    
	    return false;
	}
	
	static boolean _cmsFormatterIsFloat(int format)
	{
		return ((format >> CMM.FLOAT_SH) & 0x1) != 0;
	}
//#endif
	public void cmsDoTransform(Object inputBuffer, Object outputBuffer, int size)
	{
//#ifndef USE_LCMS_LIBRARY
		this.xform.transform(inputBuffer, outputBuffer, size);
//#else
		//TODO
//#endif
	}
	
	//Actual system functions. Just for simplicity ICC_Converter cannot be instantiated by the developer, instead the static functions will create the converter and convert the values
	
	public static short[] convertColors(ICC_Profile[] profiles, short[] src, Object[] converterCache, int cacheIndex)
	{
		int numProfiles = profiles.length;
		int[] renderingIntents = new int[numProfiles];
		// Default is perceptual
        int currRenderingIntent = ICC_Profile.icPerceptual;
        
        // render as colorimetric for output device
        if (profiles[0].getProfileClass() == ICC_Profile.CLASS_OUTPUT)
        {
            currRenderingIntent = ICC_Profile.icRelativeColorimetric;
        }
        
        // get the transforms from each profile
        for (int i = 0; i < numProfiles; i++)
        {
            // first or last profile cannot be abstract
            // if profile is abstract, the only possible way is
            // use AToB0Tag (perceptual), see ICC spec
            if (i != 0 && i != numProfiles - 1 && profiles[i].getProfileClass() == ICC_Profile.CLASS_ABSTRACT)
            {
                currRenderingIntent = ICC_Profile.icPerceptual;
            }
            
            renderingIntents[i] = currRenderingIntent;
            // use current rendering intent
            // to select LUT from the next profile (chaining)
            currRenderingIntent = profiles[i].getRenderingIntent();
        }
		
		return convertColors(profiles, renderingIntents, src, converterCache, cacheIndex);
	}
	
	public static short[] convertColors(ICC_Profile[] profiles, int[] intents, short[] src, Object[] converterCache, int cacheIndex)
	{
		//Get/Setup converter (basically the cmsHTRANSFORM)
		ICC_Converter converter;
		if(converterCache != null && converterCache.length > 0 && converterCache[cacheIndex] instanceof ICC_Converter)
		{
			converter = (ICC_Converter)converterCache[cacheIndex]; //Hopefully there the intent and profiles used to create this are the same as those in use now
		}
		else
		{
			converter = setupConverter(profiles, intents);
			if(converterCache != null && converterCache.length > 0)
			{
				//Cache so it can be used for later use
				converterCache[cacheIndex] = converter;
			}
		}
		
		if(src.length != converter.getInputChannelCount())
		{
			throw new IllegalStateException("Converter input profile does not have the same number of channels as 'src' data.");
		}
		
		short[] dst = new short[converter.getOutputChannelCount()];
		
		converter.cmsDoTransform(src, dst, 1);
		
		return dst;
	}
	
	private static ICC_Converter setupConverter(ICC_Profile[] profiles, int[] intents)
	{
		int nProfiles = profiles.length;
		
		if (nProfiles <= 0 || nProfiles > 255)
		{
			throw new IllegalArgumentException("Wrong number of profiles. 1..255 expected, " + nProfiles + " found.");
		}
		
		byte[][] profileHeaders = new byte[nProfiles][];
		
		for (int i = 0; i < nProfiles; i++)
		{
			//Cache the headers for use in other parts of the setup
			int deviceClass = CMM.getIntFromByteArray(profileHeaders[i] = profiles[i].getData(ICC_Profile.icSigHead), ICC_Profile.icHdrDeviceClass);
			if (deviceClass == ICC_Profile.icSigNamedColorClass || deviceClass == ICC_Profile.icSigLinkClass)
			{
				return null; // Unsupported named color and device link profiles
		    }
		}
		
//#ifndef USE_LCMS_LIBRARY
		return new ICC_Converter(profiles, new boolean[nProfiles], intents, new double[nProfiles], null, 0, 
				(2 << CMM.BYTES_SH) | (CMM.cmsChannelsOf(CMM.getIntFromByteArray(profileHeaders[0], ICC_Profile.icHdrColorSpace)) << CMM.CHANNELS_SH), 
				(2 << CMM.BYTES_SH) | (CMM.cmsChannelsOf(CMM.getIntFromByteArray(profileHeaders[nProfiles - 1], ICC_Profile.icHdrColorSpace)) << CMM.CHANNELS_SH), 
				CMM.cmsFLAGS_FORCE_CLUT | CMM.cmsFLAGS_CLUT_PRE_LINEARIZATION);
//#else
		return new ICC_Converter(profiles, intents);
//#endif
	}
	
//#ifndef USE_LCMS_LIBRARY
	private static interface TransformOp
	{
		public void transform(Object inputBuffer, Object outputBuffer, int size);
	}
	
	private static interface Formatter16
	{
		public Object format(short[] values, Object buffer, int[] offset, int offsetIndex, int stride);
	}
	
	private static interface FormatterFloat
	{
		public Object format(float[] values, Object buffer, int[] offset, int offsetIndex, int stride);
	}
	
	private static abstract class FormatterItem
	{
		public int type;
		public int mask;
		
		public FormatterItem(int type, int mask)
		{
			this.type = type;
			this.mask = mask;
		}
	}
	
	private static class FormatterFloatItem extends FormatterItem
	{
		public FormatterFloat frm;
		
		public FormatterFloatItem(int type, int mask, FormatterFloat frm)
		{
			super(type, mask);
			this.frm = frm;
		}
	}
	
	private static class Formatter16Item extends FormatterItem
	{
		public Formatter16 frm;
		
		public Formatter16Item(int type, int mask, Formatter16 frm)
		{
			super(type, mask);
			this.frm = frm;
		}
	}
//#endif
}
