//#preprocessor

//#ifndef USE_LCMS_LIBRARY

/*
 * File: MAT3.java
 * Version: 2.0
 * Initial Creation: Jul 4, 2010 1:23:17 PM
 *
 * SEE "Little CMS License.txt"
 */
package com.sun.pdfview.helper.graphics.color;

public class MAT3
{
	// Determinant lower than that are assumed zero (used on matrix invert)
	private static final double MATRIX_DET_TOLERANCE = 0.0001;
	
	public VEC3[] rows;
	
	public MAT3()
	{
		this(new VEC3(), new VEC3(), new VEC3());
	}
	
	public MAT3(MAT3 mat)
	{
		this(mat.rows[0], mat.rows[1], mat.rows[2]);
	}
	
	public MAT3(VEC3 r1, VEC3 r2, VEC3 r3)
	{
		rows = new VEC3[3];
		for(int i = 0; i < 3; i++)
		{
			rows[i] = new VEC3();
		}
		set(r1, r2, r3);
	}
	
	public MAT3(double[] vals, int index)
	{
		this(new VEC3(vals, index), new VEC3(vals, index + 3), new VEC3(vals, index + 6));
	}
	
	public void set(MAT3 mat)
	{
		for(int i = 0; i < 3; i++)
		{
			this.rows[i].set(mat.rows[i]);
		}
	}
	
	public void set(double[] vals, int index)
	{
		for(int i = 0; i < 3; i++)
		{
			this.rows[i].set(vals, 3 * i);
		}
	}
	
	public void set(VEC3 r1, VEC3 r2, VEC3 r3)
	{
		this.rows[0].set(r1);
		this.rows[1].set(r2);
		this.rows[2].set(r3);
	}
	
	public void setIdentity()
	{
		this.rows[0].vals[0] = 1;
		this.rows[0].vals[1] = 0;
		this.rows[0].vals[2] = 0;
		this.rows[1].vals[0] = 0;
		this.rows[1].vals[1] = 1;
		this.rows[1].vals[2] = 0;
		this.rows[2].vals[0] = 0;
		this.rows[2].vals[1] = 0;
		this.rows[2].vals[2] = 1;
	}
	
	public boolean isIdentity()
	{
		MAT3 identity = new MAT3();
		identity.setIdentity();

		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 3; j++)
			{
				if (!closeEnough(this.rows[i].vals[j], identity.rows[i].vals[j]))
				{
					return false;
				}
			}
		}
		
		return true;
	}
	
	private static boolean closeEnough(double a, double b)
	{
		return Math.abs(b - a) < (1.0 / 65535.0);
	}
	
	public double[] getAsArray()
	{
		double[] array = new double[3 * 3];
		System.arraycopy(this.rows[0].vals, 0, array, 0, 3);
		System.arraycopy(this.rows[1].vals, 0, array, 3, 3);
		System.arraycopy(this.rows[2].vals, 0, array, 6, 3);
		return array;
	}
	
	// Multiply two matrices
	public static void multiply(MAT3 r, MAT3 a, MAT3 b)
	{
		for(int i = 0; i < 3; i++)
		{
			r.rows[i].set(calcRowCol(a, b, i, 0), calcRowCol(a, b, i, 1), calcRowCol(a, b, i, 2));
		}
	}
	
	private static double calcRowCol(MAT3 a, MAT3 b, int i, int j)
	{
		return (a.rows[i].vals[0] * b.rows[0].vals[j]) + (a.rows[i].vals[1] * b.rows[1].vals[j]) + (a.rows[i].vals[2] * b.rows[2].vals[j]);
	}
	
	// Inverse of a matrix b = a^(-1)
	public static boolean inverse(MAT3 a, MAT3 b)
	{
		double c0 =  (a.rows[1].vals[1] * a.rows[2].vals[2]) - (a.rows[1].vals[2] * a.rows[2].vals[1]);
		double c1 = (-a.rows[1].vals[0] * a.rows[2].vals[2]) + (a.rows[1].vals[2] * a.rows[2].vals[0]);
		double c2 =  (a.rows[1].vals[0] * a.rows[2].vals[1]) - (a.rows[1].vals[1] * a.rows[2].vals[0]);
		
		double det = (a.rows[0].vals[0] * c0) + (a.rows[0].vals[1] * c1) + (a.rows[0].vals[2] * c2);
		
		if (Math.abs(det) < MATRIX_DET_TOLERANCE)
		{
			return false; // singular matrix; can't invert
		}
		
		b.rows[0].vals[0] = c0 / det;
		b.rows[0].vals[1] = ((a.rows[0].vals[2]*a.rows[2].vals[1]) - (a.rows[0].vals[1] * a.rows[2].vals[2])) / det;
		b.rows[0].vals[2] = ((a.rows[0].vals[1]*a.rows[1].vals[2]) - (a.rows[0].vals[2] * a.rows[1].vals[1])) / det;
		b.rows[1].vals[0] = c1 / det;
		b.rows[1].vals[1] = ((a.rows[0].vals[0]*a.rows[2].vals[2]) - (a.rows[0].vals[2] * a.rows[2].vals[0])) / det;
		b.rows[1].vals[2] = ((a.rows[0].vals[2]*a.rows[1].vals[0]) - (a.rows[0].vals[0] * a.rows[1].vals[2])) / det;
		b.rows[2].vals[0] = c2 / det;
		b.rows[2].vals[1] = ((a.rows[0].vals[1]*a.rows[2].vals[0]) - (a.rows[0].vals[0] * a.rows[2].vals[1])) / det;
		b.rows[2].vals[2] = ((a.rows[0].vals[0]*a.rows[1].vals[1]) - (a.rows[0].vals[1] * a.rows[1].vals[0])) / det;
		
		return true;
	}
	
	// Evaluate a vector across a matrix
	public static void evaluate(VEC3 r, MAT3 a, VEC3 v)
	{
		r.vals[0] = (a.rows[0].vals[0] * v.vals[0]) + (a.rows[0].vals[1] * v.vals[1]) + (a.rows[0].vals[2] * v.vals[2]);
	    r.vals[1] = (a.rows[1].vals[0] * v.vals[0]) + (a.rows[1].vals[1] * v.vals[1]) + (a.rows[1].vals[2] * v.vals[2]);
	    r.vals[2] = (a.rows[2].vals[0] * v.vals[0]) + (a.rows[2].vals[1] * v.vals[1]) + (a.rows[2].vals[2] * v.vals[2]);
	}
}

//#endif
